/****************************************************
 Statistics Online Computational Resource (SOCR)
 http://www.StatisticsResource.org
  
 All SOCR programs, materials, tools and resources are developed by and freely disseminated to the entire community.
 Users may revise, extend, redistribute, modify under the terms of the Lesser GNU General Public License
 as published by the Open Source Initiative http://opensource.org/licenses/. All efforts should be made to develop and distribute
 factually correct, useful, portable and extensible resource all available in all digital formats for free over the Internet.
  
 SOCR resources are distributed in the hope that they will be useful, but without
 any warranty; without any explicit, implicit or implied warranty for merchantability or
 fitness for a particular purpose. See the GNU Lesser General Public License for
 more details see http://opensource.org/licenses/lgpl-license.php.
  
 http://www.SOCR.ucla.edu
 http://wiki.stat.ucla.edu/socr
  It s Online, Therefore, It Exists! 
 ****************************************************/

package edu.ucla.stat.SOCR.TG_distributome;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import edu.ucla.stat.SOCR.TG_distributome.data.SOCREdge;
import edu.ucla.stat.SOCR.TG_distributome.data.SOCRFormula;
import edu.ucla.stat.SOCR.TG_distributome.data.SOCRNode;
import edu.ucla.stat.SOCR.TG_distributome.data.SOCRReference;
import edu.ucla.stat.SOCR.TG_distributome.data.TGViewerPanelInfo;
import edu.ucla.stat.SOCR.TG_distributome.editor.DistributomeEditor;

public class TGEditorApplet extends TGViewerApplet{  
		
	public boolean debug = false;
	public JButton addNewNode;
	
	DistributomeEditor  editor;
    /**CONSTRUCTOR
     * 
     */
    public TGEditorApplet() {  	   	
     super();
    }
    

    protected void initControlPanel(){
	//	System.out.println("initControlPanel get called");
		controlPanel.removeAll();
		controlPanel.setPreferredSize(new Dimension(600, 33));
		controlPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		
		highlightNodeColorChoice = new JComboBox();
		highlightEdgeColorChoice = new JComboBox();	
		zoom_level = new JComboBox();
		textEntry = new JTextField(MAXSEARCHLENGTH - 4);
		
		refreshButton = new JButton("Reload");
		refreshButton.addActionListener(this);
		toggleButton = new JButton("Toggle Control");
		toggleButton.addActionListener(this);
		
		addNewNode = new JButton("Add New Node");
		addNewNode.addActionListener(this);
	
		neighbor_level= new JComboBox();
		neighbor_level.addItem("HideNeighbor");
		neighbor_level.addItem("ShowParent");
		neighbor_level.addItem("ShowChildren");
		neighbor_level.addItem("ShowBoth");
		neighbor_level.setSelectedIndex(0);
		neighbor_level.addActionListener(this);
		
		// search text field
	
		textEntry.addKeyListener(this);					
		
		zoom_level = new JComboBox();
		zoom_level.addItem("ZoomReset");
		zoom_level.addItem("ZoomIn");
		zoom_level.addItem("ZoomOut");
		zoom_level.setSelectedIndex(0);
		zoom_level.addActionListener(this);
		
		highlightNodeColorChoice= new JComboBox();
		highlightNodeColorChoice.addItem("HighlightNodeType");
		for (int i=0; i<panelInfo.nodeType.getTypeCount(); i++){
			highlightNodeColorChoice.addItem(panelInfo.nodeType.getFullName(i));
		}
		highlightNodeColorChoice.setSelectedIndex(0);
		highlightNodeColorChoice.addActionListener(this);
		
		highlightEdgeColorChoice = new JComboBox();
		highlightEdgeColorChoice.addItem("HighlightEdgeType");
		for (int j=0; j<panelInfo.edgeType.getTypeCount(); j++){
			highlightEdgeColorChoice.addItem(panelInfo.edgeType.getFullName(j));
		}
		highlightEdgeColorChoice.setSelectedIndex(0);
		highlightEdgeColorChoice.addActionListener(this);
	
		controlPanel.add(toggleButton);
		controlPanel.add(refreshButton);
		controlPanel.add(new JLabel("Search", JLabel.RIGHT)); 
		controlPanel.add(textEntry);
		controlPanel.add(addNewNode);
		controlPanel.add(new JLabel("Double click on any node or whitespace to open editor"));
		//controlPanel.add(zoom_level);
		//controlPanel.add(neighbor_level);
	
		//controlPanel.add(highlightNodeColorChoice);
		//controlPanel.add(highlightEdgeColorChoice);
		
		this.getContentPane().repaint();
	}

    public void getXMLInputFile(){
    	super.getXMLInputFile();
		
		turnOnshowID();
		
		//System.out.println(panelInfo.getNodeCount() + ":"+panelInfo.getEdgeCount() +":"+panelInfo.nFormulas +":"+panelInfo.nRefs);
    }
    
    protected void turnOnshowID(){
	 for (int i=0; i<panelInfo.getNodeCount(); i++)
		 panelInfo.getNode(i).showID= true;
	 for (int i=0; i<panelInfo.getEdgeCount(); i++)
		 panelInfo.getEdge(i).showID= true;
	/* for (int i=0; i<panelInfo.nFormulas; i++)
		 panelInfo.getFormula(i).showID= true;
	 for (int i=0; i<panelInfo.nRefs; i++)
		 panelInfo.refs[i].showID= true;*/
    }

    
    /** ActionListener implementation method
	 * 
	 */
	public void actionPerformed(ActionEvent e) {
		super.actionPerformed(e);
	
		Object src = e.getSource();
		
		if (src == addNewNode) { 
			if (editor!=null)
				editor.dispose();
			editor = new DistributomeEditor();		  
			editor.setCodeBase(this.getCodeBase());
			editor.loadElements((SOCRNode)null);
			editor.loadElements((SOCREdge)null);
			editor.loadElements((SOCRFormula)null);
			editor.loadElements((SOCRReference)null);
			
			editor.showView();
		}
	
		// refresh button handling
		if (src == refreshButton) { 
			if (debug) System.out.println("refresh clicked");
			final Container c = this.getContentPane();
			//c.removeAll();
			c.remove(graphPanel);	
			
			panelInfo = new TGViewerPanelInfo(this);
		//	panelInfo.setFreezedStatus(false);
		
			//getInputFile();
			getXMLInputFile();
			initGraph();
			resetControlPanel();
			
			displayInitInfo();
	    	
			c.add(graphPanel,BorderLayout.CENTER);
			c.validate();
		//	panelInfo.start();
			return;
		}
	}
	
	public void doubleClickedWhitespace(){
		//System.out.println("ClickedWhitespace");
		if (editor!=null)
			editor.dispose();
		editor = new DistributomeEditor();		  
		editor.setCodeBase(this.getCodeBase());
		editor.loadElements((SOCRNode)null);
		editor.loadElements((SOCREdge)null);
		editor.loadElements((SOCRFormula)null);
		editor.loadElements((SOCRReference)null);
		
		editor.showView();
	}
	
	public void doubleClickedNode(){
		if (editor!=null)
			editor.dispose();
		editor = new DistributomeEditor();		  
		editor.setCodeBase(this.getCodeBase());
		editor.loadElements(panelInfo.getNode(panelInfo.pickedNodeIndex));
		
		int pickedNodeId = panelInfo.getNode(panelInfo.pickedNodeIndex).getId();
		//System.out.println("picked node id ="+pickedNodeId);
		
		for (int i=0; i<panelInfo.getEdgeCount(); i++)
			if (panelInfo.getEdge(i).getSourceId()==pickedNodeId||
					panelInfo.getEdge(i).getTargetId()==pickedNodeId	){
				editor.loadElements(panelInfo.getEdge(i));
			}
		
		for (int i=0; i<panelInfo.getEdgeCount(); i++)
			if (panelInfo.getFormula(i).getId()==panelInfo.getNode(panelInfo.pickedNodeIndex).getFormulaId()){
				editor.loadElements(panelInfo.getFormula(i));
			}
		
		for (int j=0; j<panelInfo.getNode(panelInfo.pickedNodeIndex).getRefCount(); j++){
			int refId= panelInfo.getNode(panelInfo.pickedNodeIndex).getRefId(j);
			for (int i=0; i<panelInfo.getRefCount(); i++)
				if (panelInfo.getRef(i).getId()==refId)
					editor.loadElements(panelInfo.getRef(i));
		}
			
		editor.showView();
	}

    
}



